@@ -499,7 +499,7 @@ LOG_LEVEL = None
 ADMIN_EMAIL = "admin@{SERVER_NAME}"  # aliased in settings.ini as "[global]admin_email"
 DATABASE_URL = DatabaseURL("DATABASE_URL")
 DATABASE_ENGINE = DATABASE_URL.engine("django.db.backends.sqlite3")
-DATABASE_NAME = DATABASE_URL.database(Path("{LOCAL_PATH}/database.db"))
+DATABASE_NAME = DATABASE_URL.database(Path("{LOCAL_PATH}/db.sqlite3"))
 DATABASE_USER = DATABASE_URL.username("")
 DATABASE_PASSWORD = DATABASE_URL.password("")
 DATABASE_HOST = DATABASE_URL.hostname("localhost")
