@@ -38,8 +38,10 @@ Since the second file overrides the first one, `TEMPLATE_DEBUG` always has the s
 
 """
 import os
+import socket
 import sys
 from typing import Any, Set
+from urllib.parse import urlparse
 
 from django.core.checks import Warning
 from django.utils.module_loading import import_string
