@@ -18,12 +18,12 @@ import logging
 import logging.handlers
 import os
 import re
+import sys
+import time
 import warnings
 from traceback import extract_stack
 from urllib.parse import urlparse
 
-import sys
-import time
 from django.core.checks.messages import Warning
 from django.core.management import color_style
 from django.utils.log import AdminEmailHandler as BaseAdminEmailHandler
@@ -32,8 +32,7 @@ from df_config.checks import settings_check_results
 
 
 class ColorizedFormatter(logging.Formatter):
-    """Used in console for applying colors to log lines, corresponding to the log level.
-    """
+    """Used in console for applying colors to log lines, corresponding to the log level."""
 
     def __init__(self, *args, **kwargs):
         self.style = color_style()
@@ -108,8 +107,9 @@ class ServerFormatter(logging.Formatter):
 # noinspection PyClassHasNoInit
 class AdminEmailHandler(BaseAdminEmailHandler):
     """Enhance the AdminEmailHandler provided by Django:
-     Does not try to send email if `settings.EMAIL_HOST` is not set.
-     Also limits the mail rates to avoid to spam the poor admins."""
+    Does not try to send email if `settings.EMAIL_HOST` is not set.
+    Also limits the mail rates to avoid to spam the poor admins.
+    """
 
     _previous_email_time = None
     min_interval = 600
@@ -143,8 +143,7 @@ class AdminEmailHandler(BaseAdminEmailHandler):
 
 
 class RemoveDuplicateWarnings(logging.Filter):
-    """Displays py.warnings messages unless the same warning was already sent.
-    """
+    """Displays py.warnings messages unless the same warning was already sent."""
 
     def __init__(self, name=""):
         super().__init__(name=name)
@@ -158,6 +157,24 @@ class RemoveDuplicateWarnings(logging.Filter):
         return result
 
 
+class SlowQueriesFilter(logging.Filter):
+    """Filter slow queries and attach stack_info."""
+
+    def __init__(self, name="", slow_query_duration_in_s=1.0):
+        super().__init__(name=name)
+        self.slow_query_duration_in_s = slow_query_duration_in_s
+
+    def filter(self, record):
+        duration = getattr(record, "duration", 0)
+        if duration > self.slow_query_duration_in_s:
+            # Same as in _log for when stack_info=True is used.
+            # noinspection PyTypeChecker
+            fn, lno, func, sinfo = logging.Logger.findCaller(None, True)
+            record.stack_info = sinfo
+            return True
+        return False
+
+
 # noinspection PyMethodMayBeStatic
 class LogConfiguration:
     """Generate a log configuration depending on a few parameters:
@@ -169,14 +186,16 @@ class LogConfiguration:
 
     Required values in the `settings_dict`:
 
-    *  `DEBUG`: `True` or `False`
-    *  `DF_MODULE_NAME`: your project name, used to determine log filenames,
-    *  `LOG_DIRECTORY`: dirname where log files are written (!),
-    *  `LOG_REMOTE_URL`: examples: "syslog+tcp://localhost:514/user", "syslog:///local7"
+    * `DEBUG`: `True` or `False`
+    * `DF_MODULE_NAME`: your project name, used to determine log filenames,
+    * `LOG_DIRECTORY`: dirname where log files are written (!),
+    * `LOG_LEVEL`: one of "debug", "info", "warning", "error", "critical"
+    * `LOG_REMOTE_URL`: examples: "syslog+tcp://localhost:514/user", "syslog:///local7"
          "syslog:///dev/log/daemon", "logd:///project_name"
-    *  `LOG_REMOTE_ACCESS`: also send HTTP requests to syslog/journald
-    *  `SERVER_NAME`: the public name of the server (like "www.example.com")
-    *  `SERVER_PORT`: the public port (probably 80 or 443)
+    * `LOG_REMOTE_ACCESS`: also send HTTP requests to syslog/journald
+    * `LOG_SLOW_QUERY_DURATION_IN_S`: log requests that takes more than this time
+    * `SERVER_NAME`: the public name of the server (like "www.example.com")
+    * `SERVER_PORT`: the public port (probably 80 or 443)
     * `LOG_EXCLUDED_COMMANDS`: Django commands that do not write logs
     """
 
@@ -185,18 +204,24 @@ class LogConfiguration:
         "DF_MODULE_NAME",
         "LOG_DIRECTORY",
         "LOG_REMOTE_URL",
+        "LOG_SLOW_QUERY_DURATION_IN_S",
         "LOG_REMOTE_ACCESS",
         "SERVER_NAME",
         "SERVER_PORT",
         "LOG_EXCLUDED_COMMANDS",
         "LOG_LEVEL",
     ]
-    # for loggers that only show INFO in debug mode, or WARN in INFO, and so on:
-    _level_up = {"DEBUG": "INFO", "INFO": "WARN", "WARN": "ERROR", "ERROR": "CRITICAL"}
+    # for loggers that only show INFO in debug mode, or WARNING in INFO, and so on:
+    _level_up = {
+        "DEBUG": "INFO",
+        "INFO": "WARNING",
+        "WARNING": "ERROR",
+        "ERROR": "CRITICAL",
+    }
     # set of always INFO loggers that are not propagated to root
     _level_access = {
         "DEBUG": "INFO",
-        "WARN": "INFO",
+        "WARNING": "INFO",
         "ERROR": "INFO",
         "CRITICAL": "INFO",
     }
@@ -211,14 +236,27 @@ class LogConfiguration:
     problem_loggers = {
         "django": _level_up,
         "django.db": _level_up,
-        "django.db.backends": _level_up,
+        "django.db.backends": {
+            "DEBUG": "DEBUG",
+            "INFO": "DEBUG",
+            "WARNING": "DEBUG",
+            "ERROR": "ERROR",
+            "CRITICAL": "CRITICAL",
+        },
         "django.request": {},
-        "django.security": _level_up,
+        "django.security": {},
         "df_websockets.signals": {},
         "gunicorn.error": {},
         "pip.vcs": _level_up,
         "py.warnings": _level_up,
     }
+    compat_log_levels = {
+        "WARN": "WARNING",
+        "CRIT": "CRITICAL",
+        "EMERGENCY": "CRITICAL",
+        "ALERT": "CRITICAL",
+        "NOTICE": "INFO",
+    }
 
     # all loggers that will be defined
     # values are dict to map chosen log level (by the admin) to the log level
@@ -235,6 +273,7 @@ class LogConfiguration:
         self.server_name = None
         self.log_level = None
         self.server_port = None
+        self.slow_query_duration_in_s = None
         self.excluded_commands = {}
         self.stdout = stdout or sys.stdout
         self.stderr = stderr or sys.stderr
@@ -246,20 +285,24 @@ class LogConfiguration:
         self.module_name = settings_dict["DF_MODULE_NAME"]
         self.server_name = settings_dict["SERVER_NAME"]
         self.server_port = settings_dict["SERVER_PORT"]
+        self.slow_query_duration_in_s = settings_dict["LOG_SLOW_QUERY_DURATION_IN_S"]
         self.excluded_commands = settings_dict["LOG_EXCLUDED_COMMANDS"]
         if settings_dict["LOG_LEVEL"]:
             log_level = settings_dict["LOG_LEVEL"].upper()
         elif settings_dict["DEBUG"]:
             log_level = "DEBUG"
         else:
-            log_level = "WARN"
+            log_level = "WARNING"
+        log_level = self.compat_log_levels.get(log_level, log_level)
         self.formatters = self.get_default_formatters()
         self.filters = self.get_default_filters()
         self.loggers = self.get_default_loggers()
         self.handlers = self.get_default_handlers()
         self.root = self.get_default_root()
         self.log_suffix = self.get_smart_command_name(
-            self.module_name, argv, self.excluded_commands,
+            self.module_name,
+            argv,
+            self.excluded_commands,
         )
         self.log_directory = settings_dict["LOG_DIRECTORY"]
         config = {
@@ -276,7 +319,6 @@ class LogConfiguration:
             self.loggers[logger]["level"] = levels.get(log_level, log_level)
         for logger, levels in self.access_loggers.items():
             self.loggers[logger]["level"] = levels.get(log_level, log_level)
-
         if settings_dict["DEBUG"]:
             warnings.simplefilter("always", DeprecationWarning)
             logging.captureWarnings(True)
@@ -295,9 +337,14 @@ class LogConfiguration:
                 self.add_handler(logger, "access", level="DEBUG", formatter="nocolor")
             has_handler = True
 
-        has_handler = self.add_remote_collector(
-            settings_dict["LOG_REMOTE_URL"], settings_dict["LOG_REMOTE_ACCESS"], level=log_level
-        ) or has_handler
+        has_handler = (
+            self.add_remote_collector(
+                settings_dict["LOG_REMOTE_URL"],
+                settings_dict["LOG_REMOTE_ACCESS"],
+                level=log_level,
+            )
+            or has_handler
+        )
         if not has_handler or not self.log_suffix:
             # (no file or interactive command) and no logd/syslog => we print to the console (like the debug mode)
             self.add_handler("ROOT", "stdout", level=log_level, formatter="colorized")
@@ -311,7 +358,7 @@ class LogConfiguration:
     def __repr__(self):
         return "%s.%s" % (self.__module__, "log_configuration")
 
-    def add_remote_collector(self, log_remote_url, log_remote_access, level="WARN"):
+    def add_remote_collector(self, log_remote_url, log_remote_access, level="WARNING"):
         has_handler = False
         if not log_remote_url:
             return has_handler
@@ -322,27 +369,43 @@ class LogConfiguration:
             address, facility, socktype = self.parse_syslog_url(
                 parsed_log_url, scheme, device, facility_name
             )
-            self.add_handler(
-                "ROOT",
-                "syslog",
-                level=level,
-                address=address,
-                facility=facility,
-                socktype=socktype,
-                formatter="nocolor",
-            )
+            kwargs = {
+                "address": address,
+                "facility": facility,
+                "socktype": socktype,
+                "formatter": "nocolor",
+            }
+            self.add_handler("ROOT", "syslog", level=level, **kwargs)
             if log_remote_access:
                 for logger in self.access_loggers:
-                    self.add_handler(
-                        logger,
-                        "syslog",
-                        level="DEBUG",
-                        address=address,
-                        facility=facility,
-                        socktype=socktype,
-                        formatter="nocolor",
-                    )
+                    self.add_handler(logger, "syslog", level="DEBUG", **kwargs)
             has_handler = True
+        elif scheme == "loki" or scheme == "lokis":
+            url = f"http://{parsed_log_url.hostname}"
+            if scheme == "lokis":
+                url = f"https://{parsed_log_url.hostname}"
+            if parsed_log_url.port:
+                url += f":{parsed_log_url.port}"
+            if parsed_log_url.path:
+                url += parsed_log_url.path
+            if parsed_log_url.query:
+                url += f"?{parsed_log_url}"
+            auth = None
+            if parsed_log_url.username and parsed_log_url.password:
+                auth = (parsed_log_url.username, parsed_log_url.password)
+            kwargs = {"url": url, "auth": auth}
+            self.add_handler("ROOT", "loki", level=level, **kwargs)
+            if log_remote_access:
+                for logger in self.access_loggers:
+                    self.add_handler(logger, "loki", level="DEBUG", **kwargs)
+            has_handler = True
+        else:
+            warning = Warning(
+                "The only known schemes for remote logging are syslog, syslog+tcp, loki or lokis.",
+                hint=None,
+                obj="configuration",
+            )
+            settings_check_results.append(warning)
         return has_handler
 
     def parse_syslog_url(self, parsed_log_url, scheme, device, facility_name):
@@ -404,26 +467,40 @@ class LogConfiguration:
         }
 
     def get_default_filters(self):
-        return {
+        filters = {
             "remove_duplicate_warnings": {
                 "()": "df_config.guesses.log.RemoveDuplicateWarnings"
-            }
+            },
         }
+        if self.slow_query_duration_in_s:
+            filters["slow_queries"] = {
+                "()": "df_config.guesses.log.SlowQueriesFilter",
+                "slow_query_duration_in_s": self.slow_query_duration_in_s,
+            }
+
+        return filters
 
     def get_default_root(self):
-        return {"handlers": [], "level": "WARN"}
+        return {"handlers": [], "level": "WARNING"}
 
     def get_default_loggers(self):
         loggers = {}
         for logger in self.problem_loggers:
             loggers[logger] = {"handlers": [], "level": "DEBUG", "propagate": True}
+        if self.slow_query_duration_in_s:
+            loggers["django.db.backends"]["filters"] = ["slow_queries"]
         loggers["py.warnings"]["filters"] = ["remove_duplicate_warnings"]
         for logger in self.access_loggers:
             loggers[logger] = {"handlers": [], "level": "DEBUG", "propagate": False}
         return loggers
 
     def add_handler(
-        self, logger: str, filename: str, level: str = "WARN", formatter=None, **kwargs
+        self,
+        logger: str,
+        filename: str,
+        level: str = "WARNING",
+        formatter=None,
+        **kwargs,
     ):
         """Add a handler to a logger.
         The name of the added handler is unique, so the definition of the handler is also add if required.
@@ -432,94 +509,27 @@ class LogConfiguration:
         filename: can be a filename or one of the following special values: "stderr", "stdout", "logd", "syslog"
         """
         if filename == "stderr":
-            handler_name = "%s.%s" % (filename, level.lower())
-            if formatter in ("django.server", "colorized") and not self.stderr.isatty():
-                formatter = None
-            elif formatter:
-                handler_name += ".%s" % formatter
-            handler = {
-                "class": "logging.StreamHandler",
-                "level": level,
-                "stream": "ext://sys.stderr",
-                "formatter": formatter,
-            }
+            handler, handler_name = self.add_handler_stderr(filename, formatter, level)
         elif filename == "stdout":
-            handler_name = "%s.%s" % (filename, level.lower())
-            if formatter in ("django.server", "colorized") and not self.stdout.isatty():
-                formatter = None
-            elif formatter:
-                handler_name += ".%s" % formatter
-            handler = {
-                "class": "logging.StreamHandler",
-                "level": level,
-                "stream": "ext://sys.stdout",
-                "formatter": formatter,
-            }
+            handler, handler_name = self.add_handler_stdout(filename, formatter, level)
+        elif filename == "loki":
+            handler, handler_name = self.add_handler_loki(
+                logger, filename, level, kwargs
+            )
         elif filename == "syslog":
-            handler_name = "%s.%s" % (filename, level.lower())
+            handler_name = f"{filename}.{level.lower()}"
             handler = {"class": "logging.handlers.SysLogHandler", "level": level}
             handler.update(kwargs)
         elif filename == "logd":
-            try:
-                # noinspection PyUnresolvedReferences,PyPackageRequirements
-                import systemd.journal
-            except ImportError:
-                warning = Warning(
-                    "Unable to import systemd.journal (required to log with journlad)",
-                    hint=None,
-                    obj="configuration",
-                )
-                settings_check_results.append(warning)
-                # replace logd by writing to a plain-text log
-                self.add_handler(logger, level.lower(), level=level)
-                return
-            handler_name = "%s.%s" % (filename, level.lower())
-            handler = {"class": "systemd.journal.JournalHandler", "level": level}
-            handler.update(kwargs)
-        else:  # basename of a plain-text log
-            log_directory = os.path.normpath(self.log_directory)
-            if not os.path.isdir(log_directory):
-                if not self.log_directory_warning:
-                    warning = Warning(
-                        'Missing directory "%s"'
-                        % log_directory,
-                        hint=None,
-                        obj=log_directory,
-                    )
-                    settings_check_results.append(warning)
-                    self.log_directory_warning = True
-                self.add_handler(logger, "stdout", level=level, **kwargs)
-                return
-            basename = "%s-%s.log" % (self.log_suffix, filename)
-            log_filename = os.path.join(log_directory, basename)
-            try:
-                remove = not os.path.exists(log_filename)
-                open(log_filename, "a").close()  # ok, we can write
-                if (
-                    remove
-                ):  # but if this file did not exist, we remove it to avoid lot of empty log files...
-                    os.remove(log_filename)
-            except PermissionError:
-                warning_ = Warning(
-                    'Unable to write logs in "%s". Unsufficient rights?'
-                    % log_directory,
-                    hint=None,
-                    obj=log_directory,
-                )
-                settings_check_results.append(warning_)
-                self.add_handler(logger, "stdout", level=level, **kwargs)
-                return
-            handler_name = "%s.%s" % (self.log_suffix, filename)
-            handler = {
-                "class": "logging.handlers.RotatingFileHandler",
-                "maxBytes": 1000000,
-                "backupCount": 3,
-                "formatter": "nocolor",
-                "filename": log_filename,
-                "level": level,
-                "delay": True,
-            }
-
+            handler, handler_name = self.add_handler_logd(
+                logger, filename, level, kwargs
+            )
+        elif self.log_directory:  # basename of a plain-text log
+            handler, handler_name = self.add_handler_directory(
+                logger, filename, level, kwargs
+            )
+        else:
+            handler, handler_name = None, None
         if not handler_name:
             return
         if handler_name not in self.handlers:
@@ -531,11 +541,119 @@ class LogConfiguration:
         if handler_name not in target["handlers"]:
             target["handlers"].append(handler_name)
 
+    def add_handler_loki(self, logger, filename, level, kwargs):
+        try:
+            # noinspection PyUnresolvedReferences,PyPackageRequirements
+            import logging_loki
+        except ImportError:
+            warning = Warning(
+                "Unable to import logging_loki (required to log to Loki)",
+                hint=None,
+                obj="configuration",
+            )
+            settings_check_results.append(warning)
+            # replace loki by writing to a plain-text log
+            self.add_handler(logger, level.lower(), level=level)
+            return None, None
+        handler_name = f"{filename}.{level.lower()}"
+        handler = {"class": "df_config.extra.loki.LokiHandler", "level": level}
+        handler.update(kwargs)
+        return handler, handler_name
+
+    def add_handler_logd(self, logger, filename, level, kwargs):
+        try:
+            # noinspection PyUnresolvedReferences,PyPackageRequirements
+            import systemd.journal
+        except ImportError:
+            warning = Warning(
+                "Unable to import systemd.journal (required to log with journlad)",
+                hint=None,
+                obj="configuration",
+            )
+            settings_check_results.append(warning)
+            # replace logd by writing to a plain-text log
+            self.add_handler(logger, level.lower(), level=level)
+            return None, None
+        handler_name = f"{filename}.{level.lower()}"
+        handler = {"class": "systemd.journal.JournalHandler", "level": level}
+        handler.update(kwargs)
+        return handler, handler_name
+
+    def add_handler_directory(self, logger, filename, level, kwargs):
+        log_directory = os.path.normpath(self.log_directory)
+        if not os.path.isdir(log_directory):
+            if not self.log_directory_warning:
+                warning = Warning(
+                    f'Missing directory "{log_directory}"',
+                    hint=None,
+                    obj=log_directory,
+                )
+                settings_check_results.append(warning)
+                self.log_directory_warning = True
+            self.add_handler(logger, "stdout", level=level, **kwargs)
+            return None, None
+        basename = f"{self.log_suffix}-{filename}.log"
+        log_filename = os.path.join(log_directory, basename)
+        try:
+            remove = not os.path.exists(log_filename)
+            open(log_filename, "a").close()  # ok, we can write
+            if (
+                remove
+            ):  # but if this file did not exist, we remove it to avoid lot of empty log files...
+                os.remove(log_filename)
+        except PermissionError:
+            warning_ = Warning(
+                f'Unable to write logs in "{log_directory}". Unsufficient rights?',
+                hint=None,
+                obj=log_directory,
+            )
+            settings_check_results.append(warning_)
+            self.add_handler(logger, "stdout", level=level, **kwargs)
+            return None, None
+        handler_name = "%s.%s" % (self.log_suffix, filename)
+        handler = {
+            "class": "logging.handlers.RotatingFileHandler",
+            "maxBytes": 1000000,
+            "backupCount": 3,
+            "formatter": "nocolor",
+            "filename": log_filename,
+            "level": level,
+            "delay": True,
+        }
+        return handler, handler_name
+
+    def add_handler_stdout(self, filename, formatter, level):
+        handler_name = f"{filename}.{level.lower()}"
+        if formatter in ("django.server", "colorized") and not self.stdout.isatty():
+            formatter = None
+        elif formatter:
+            handler_name += f".{formatter}"
+        handler = {
+            "class": "logging.StreamHandler",
+            "level": level,
+            "stream": "ext://sys.stdout",
+            "formatter": formatter,
+        }
+        return handler, handler_name
+
+    def add_handler_stderr(self, filename, formatter, level):
+        handler_name = f"{filename}.{level.lower()}"
+        if formatter in ("django.server", "colorized") and not self.stderr.isatty():
+            formatter = None
+        elif formatter:
+            handler_name += f".{formatter}"
+        handler = {
+            "class": "logging.StreamHandler",
+            "level": level,
+            "stream": "ext://sys.stderr",
+            "formatter": formatter,
+        }
+        return handler, handler_name
+
     @staticmethod
     def get_smart_command_name(module_name, argv, excluded_commands=None):
         """Return a "smart" name for the current command line.
         If it's an interactive Django command (think to "migrate"), returns None
-        It it's the celery worker process, specify the running queues in the name
         Otherwise, add the Django command in the name.
 
         :param module_name:
