@@ -1,7 +1,7 @@
 # ##############################################################################
 #  This file is part of df_config                                              #
 #                                                                              #
-#  Copyright (C) 2020 Matthieu Gallet <github@19pouces.net>                    #
+#  Copyright (C) 2020 Matthieu Gallet <df_config@19pouces.net>                    #
 #  All Rights Reserved                                                         #
 #                                                                              #
 #  You may use, distribute and modify this code under the                      #
@@ -49,8 +49,7 @@ def ensure_dir(path, parent=True):
 
     """
     dirname = os.path.dirname(path) if parent else path
-    if not os.path.isdir(dirname):
-        os.makedirs(dirname)
+    os.makedirs(dirname, exist_ok=True)
     return path
 
 
@@ -108,7 +107,7 @@ def get_view_from_string(view_as_str):
 
 
 class ChunkReader:
-    """ read a file object in chunks of the given size.
+    """read a file object in chunks of the given size.
 
     Return an iterator of data
 
@@ -210,6 +209,7 @@ def send_file(
     :param force_download: always force the client to download the file.
     :param attachment_filename: filename used in the "Content-Disposition" header (when used)
     :param chunk_size: size of chunks for large files. Useful at least for unittests
+    :param etag: ETag header to add to the response
     :rtype: :class:`django.http.response.StreamingHttpResponse` or :class:`django.http.response.HttpResponse`
     """
     if mimetype is None:
@@ -260,6 +260,7 @@ def send_file(
     if settings.USE_X_SEND_FILE and not ranges:
         response = HttpResponse(content_type=mimetype)
         response["X-SENDFILE"] = filepath
+        response["Content-Length"] = str(content_size)
     elif settings.X_ACCEL_REDIRECT and not ranges:
         for dirpath, alias_url in settings.X_ACCEL_REDIRECT:
             dirpath = os.path.abspath(dirpath)
@@ -268,6 +269,7 @@ def send_file(
                 response["X-Accel-Redirect"] = os.path.join(
                     alias_url, os.path.relpath(filepath, dirpath)
                 )
+                response["Content-Length"] = str(content_size)
                 break
     if response is None:
         fileobj = open(filepath, "rb")
